# article_randomiser.py

# This script will create three directories: train, devel, and test, containing
# approximately the proportions specified of the total text, in a random
# order.  The default proportions are 60/20/20%.

# The articles are assumed to be separated by an identifying line.
# The default is that accompanying the Westbury Lab Wikipedia Corpus.
# The Westbury Lab Corpus consists of a single file containing all and
# only the text of the articles of Wikipedia.  Each article begins
# with a title on a single line, and ends with
# "---END.OF.DOCUMENT---\n\n"

import sys, random

class Randomiser():
    def __init__(self, article_file_obj, train_file_obj, devel_file_obj, test_file_obj, rand_obj, proportions=[.6,.2,.2], article_separation_line="---END.OF.DOCUMENT---\n") :
        self.article_file_obj = article_file_obj
        self.train_file_obj = train_file_obj
        self.devel_file_obj = devel_file_obj
        self.test_file_obj = test_file_obj
        self.proportions = proportions
        self.rand_obj = rand_obj
        assert sum(self.proportions) == 1, proportions
        self.article_separation_line = article_separation_line

    def choose_outfile(self):
        x = self.rand_obj.random()
        if x < self.proportions[0]: return self.train_file_obj
        if x < self.proportions[0] + self.proportions[1]: return self.devel_file_obj
        return self.test_file_obj
        
    def randomise(self):
        blank_between_articles = False
        outfile = self.choose_outfile()
        for line in self.article_file_obj:
            if line == self.article_separation_line:
                outfile.write(line)
                blank_between_articles = True
            elif blank_between_articles:
                outfile = self.choose_outfile()
                blank_between_articles = False
            else:
                outfile.write(line)


if __name__ == '__main__':
    article_file_obj = sys.stdin
    train_file_obj = open("train", 'w')
    devel_file_obj = open("devel", 'w')
    test_file_obj = open("test", 'w')
    r = random.Random(9)
    article_separation_line = sys.argv[1] + '\n'
    proportions = [float(s) for s in sys.argv[2:5]]
    ar = Randomiser(article_file_obj, train_file_obj, devel_file_obj, test_file_obj, r, proportions, article_separation_line)
    ar.randomise()



# L. Amber Wilcox-O'Hearn 2011
# test_article_randomiser.py

from code import article_randomiser
import unittest, StringIO, random, subprocess, bz2, os


class ArticleRandomiserTest(unittest.TestCase):

    def test_randomise(self):

        r = random.Random(999)

        article_separation_line = "---END.OF.DOCUMENT---\n"

        a1 = ["Anarchism.\n", "Anarchism is a political philosophy which considers the state undesirable, unnecessary and harmful, and instead promotes a stateless society, or anarchy. It seeks to diminish or even abolish authority in the conduct of human relations. Anarchists may widely disagree on what additional criteria are required in anarchism. \"The Oxford Companion to Philosophy\" says, \"there is no single defining position that all anarchists hold, and those considered anarchists at best share a certain family resemblance.\"", "---END.OF.DOCUMENT---\n"]
        a2 = ["Hidehiko Shimizu.\n", "Hidehiko Shimizu (born 4 November 1954) is a former Japanese football player. He has played for Nissan Motors.\n", "---END.OF.DOCUMENT---\n"]
        a3 = ["Some other thing.\n", "this\n", "could\n", "be a line or three.\n", "---END.OF.DOCUMENT---\n"]
        a4 = ["Finally.\n", "Another one.\n", "---END.OF.DOCUMENT---\n"]

        newline_list = ["\n"]

        article_file_obj = a1 + newline_list + a2 + newline_list + a3 + newline_list + a4

        train_file_obj = StringIO.StringIO()
        devel_file_obj = StringIO.StringIO()
        test_file_obj = StringIO.StringIO()

        ar = article_randomiser.Randomiser(article_file_obj, train_file_obj, devel_file_obj, test_file_obj, r)
        ar.randomise()
        assert train_file_obj.getvalue() == "".join(a2+a4), "".join(a2+a4) + train_file_obj.getvalue()
        assert devel_file_obj.getvalue() == "".join(a1), "".join(a1) + devel_file_obj.getvalue()
        assert test_file_obj.getvalue() == "".join(a3),  "".join(a3) + test_file_obj.getvalue() 

    def test_command_line(self):

        randomize = subprocess.Popen(['python', 'code/article_randomiser.py', "---END.OF.DOCUMENT---", '.5', '.3', '.2'], stdin=-1, stdout=-1, stderr=-1 )
        test_data_reader = bz2.BZ2File('code/test/data/small_westbury.txt.bz2', 'r')
        randomize.communicate(input=test_data_reader.read())
        self.assertEqual(randomize.returncode, 0)
        num_train = subprocess.Popen(['grep', '-c', 'END', 'train'], stdout=-1)
        self.assertEqual(int(num_train.stdout.read()), 11), num_train
        num_devel = subprocess.Popen(['grep', '-c', 'END', 'devel'], stdout=-1)
        self.assertEqual(int(num_devel.stdout.read()), 5), num_devel
        num_test = subprocess.Popen(['grep', '-c', 'END', 'test'], stdout=-1)
        self.assertEqual(int(num_test.stdout.read()), 2), num_test
        os.remove('train')
        os.remove('devel')
        os.remove('test')

if __name__ == '__main__':
    unittest.main()



# utils.py

import codecs, bz2, gzip

def open_with_unicode(file_name, compression_type, mode):
    assert compression_type in [None, 'gzip', 'bzip2']
    assert mode in ['r', 'w']
    if compression_type == None:
        if mode == 'r':
            return codecs.getreader('utf-8')(open(file_name, mode))
        elif mode == 'w':
            return codecs.getwriter('utf-8')(open(file_name, mode))
    elif compression_type == 'gzip':
        if mode == 'r':
            return codecs.getreader('utf-8')(gzip.GzipFile(file_name, mode))
        elif mode == 'w':
            return codecs.getwriter('utf-8')(gzip.GzipFile(file_name, mode))
    elif compression_type == 'bzip2':
        if mode == 'r':
            return codecs.getreader('utf-8')(bz2.BZ2File(file_name, mode))
        elif mode == 'w':
            return codecs.getwriter('utf-8')(bz2.BZ2File(file_name, mode))




#!/usr/bin/env python

from distutils.core import setup

setup(name='recluse',
      version='0.1.1',
      description='Reproducible Experimentation for Computational Linguistics Use',
      long_description=open('README.rst').read(),
      author='L. Amber Wilcox-O\'Hearn',
      author_email='amber@cs.toronto.edu',
      url='https://github.com/lamber/recluse',
      packages=['code', 'code.test'],
      license='COPYING',
     )
